/*----------------------------------------------------------*
   sci3.c for H8/3694F additional library v3.1
                                       Last Edit '06 07/31
   Copyright (c) 1999-2006 BestTechnology CO.,LTD.
 *----------------------------------------------------------*/

#include "3694.h"

#define rs_init     SCI3_INIT
#define rs_rx_buff  SCI3_IN_DATA_CHECK
#define rs_tx_buff  SCI3_OUT_DATA_CHECK
#define rs_rx_purge SCI3_IN_DATA_CLEAR
#define rs_tx_purge SCI3_OUT_DATA_CLEAR
#define rs_putc     SCI3_OUT_DATA
#define rs_puts     SCI3_OUT_STRING
#define rs_putsb    SCI3_OUT_STRINGB
#define rs_getc     SCI3_IN_DATA
#define rs_gets     SCI3_IN_STRING
#define rs_intsci   int_sci3

#define FIFO        sci3fifo
#define SCI         SCI3
#define SCR         SCR3

typedef struct {
  struct {
    char  *buf;
    short wp;
    short rp;
    short maxlen;
  } tx;
  struct {
    char  *buf;
    short wp;
    short rp;
    short maxlen;
  } rx;
} SCIFIFO;

volatile SCIFIFO FIFO;

/*-----------------------------------------------*/
/* ʐM֌W                                      */
/*-----------------------------------------------*/
// SCI
void rs_init (TBaudRate b, char *txbuf, short txl, char *rxbuf, short rxl)
{
  volatile short i;

  // Mf[^pϐ
  FIFO.tx.buf = txbuf;
  FIFO.rx.buf = rxbuf;
  FIFO.tx.maxlen = txl;
  FIFO.rx.maxlen = rxl;
  FIFO.tx.wp = FIFO.tx.rp = 0;
  FIFO.rx.wp = FIFO.rx.rp = 0;

  // RS232CR~jP[Vݒ
  IO.PMR1.BIT.TXD = 1 ;       // P22TXD[qƂĎgݒ
  SCI.SCR.BYTE = SCI.SMR.BYTE = 0;
  SCI.BRR = b;                // {[[gݒ
  for (i = 0; i < 100; i++) ; // _~[EFCg
  SCI.SSR.BYTE;
  SCI.SSR.BYTE = 0x80;        // Xe[^XNA
  SCI.SCR.BYTE = 0xf0;        // M荞݋
}

// Mobt@`FbN
short rs_tx_buff (void)
{
  short s;
  if ((s = (FIFO.tx.rp - FIFO.tx.wp)) <= 0)
    s += FIFO.tx.maxlen;
  return s - 1;
}

// Mobt@`FbN
short rs_rx_buff (void)
{
  short s;
  if ((s = (FIFO.rx.wp - FIFO.rx.rp)) < 0) s += FIFO.rx.maxlen;
  return s;
}

// Mobt@NA
void rs_rx_purge (void)
{
  SCI.SCR.BIT.RIE = 0;
  FIFO.rx.wp = FIFO.rx.rp = 0;
  SCI.SCR.BIT.RIE = 1;
}

// Mobt@NA
void rs_tx_purge (void)
{
  SCI.SCR.BIT.TIE = 0;
  FIFO.tx.wp = FIFO.tx.rp = 0;
}

// 1oCgM
void rs_putc (char c)
{
  volatile short t;

  t = (FIFO.tx.wp + 1) % FIFO.tx.maxlen;
  while (t == FIFO.tx.rp) ;
  FIFO.tx.buf[FIFO.tx.wp] = c;
  FIFO.tx.wp = t;
  SCI.SCR.BIT.TIE = 1;
}

// eLXgM (null^[~l[V)
void rs_puts (char *st)
{
  while (*st) {
    rs_putc (*st);
    if (*st++ == '\n') rs_putc ('\r');
  }
}

// oCi񑗐M (TCYw)
void rs_putsb (char *st, short l)
{
  short i;

  for (i = 0; i < l; i++) rs_putc (*st++);
}

// Mf[^obt@1ǂݏo
char rs_getc(void)
{
  char c;

  while (FIFO.rx.rp == FIFO.rx.wp) ;
  c = FIFO.rx.buf[FIFO.rx.rp++];
  FIFO.rx.rp %= FIFO.rx.maxlen;
  return c;
}

// M
short rs_gets (char *s, short len)
{
  short i = 0;
  char c;
  while (1) {
    c = rs_getc ();
    if ((c == '\n') || (c == '\r')) {
      s[i] = '\0'; if (i != 0) i++;
      rs_putc ('\n');
      break;
    } else if ((i > 0) && (c == '\b')) {
      rs_puts ("\b \b");
      i--;
    } else if ((i < len) && (i >= 0) && (c >= ' ') && (c <= '~')) {
      s[i++] = (char)c;
      rs_putc (c);
    } else rs_putc ('\a');
  }
  return i;
}

// SCI荞݊֐(wb_t@CɃvg^Cv錾ς)
void rs_intsci (void)
{
  char c;
  volatile short t;

  if (SCI.SSR.BYTE & 0x38) SCI.SSR.BYTE &= ~0x38;
  // M
  if (SCI.SSR.BIT.RDRF) {
    c = SCI.RDR;
    SCI.SSR.BIT.RDRF = 0;
    t = (FIFO.rx.wp + 1) % FIFO.rx.maxlen;
    FIFO.rx.buf[FIFO.rx.wp] = c;
    if (t != FIFO.rx.rp) FIFO.rx.wp = t;
  }
  // M
  if (SCI.SSR.BIT.TDRE) {
    if (FIFO.tx.wp != FIFO.tx.rp) {
      SCI.TDR = FIFO.tx.buf[FIFO.tx.rp++];
      FIFO.tx.rp %= FIFO.tx.maxlen;
    } else
      SCI.SCR.BIT.TIE = 0;
  }
}
